{**************************************************************************************************}
{                                                                                                  }
{  TWinPaintBox - win control paint box                                                            }
{                                                                                                  }
{  The contents of this file are subject to the Y Library Public License Version 1.0 (the          }
{  "License"); you may not use this file except in compliance with the License. You may obtain a   }
{  copy of the License at http://delphi.pjh2.de/                                                   }
{                                                                                                  }
{  Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF  }
{  ANY KIND, either express or implied. See the License for the specific language governing        }
{  rights and limitations under the License.                                                       }
{                                                                                                  }
{  The Original Code is: YclTextDatas.pas.                                                         }
{  The Initial Developer of the Original Code is Peter J. Haas (libs@pjh2.de). Portions created    }
{  by Peter J. Haas are Copyright (C) 1999-2005 Peter J. Haas. All Rights Reserved.                }
{                                                                                                  }
{  Contributor(s):                                                                                 }
{                                                                                                  }
{  You may retrieve the latest version of this file at the homepage of Peter J. Haas, located at   }
{  http://delphi.pjh2.de/                                                                          }
{                                                                                                  }
{**************************************************************************************************}

// For history see end of file

unit WinPntBx;

interface
uses
  Windows, Messages, MMSystem, Classes, Controls, Graphics;

type
  TWinPaintBox = class(TCustomControl)
  private
    FOnPaint: TNotifyEvent;
  protected
    procedure Paint; override;
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
  public
    constructor Create(AOwner: TComponent); override;
    procedure InvalidateControl;
    property Canvas;
  published
    property Align;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property OnStartDrag;
  end;

const
  WM_PaintNextFrame = WM_USER + 0;
  WM_SetNextFrame   = WM_USER + 1;

type
  TAnimationPaintBox = class(TWinPaintBox)
  private
    FTimerID: Cardinal;
    FPaintLock: Boolean;       // True, wenn gerade ein Frame gezeichnet wird
    FOnPaintNextFrame: TNotifyEvent;
    FOnSetNextFrame: TNotifyEvent;
    procedure TimerStart(FramePeriod: Cardinal);
    procedure TimerStop;
    function GetIsActive: Boolean;
  protected
    procedure WMPaintNextFrame(var Message: TMessage); message WM_PaintNextFrame;
    procedure WMSetNextFrame(var Message: TMessage); message WM_SetNextFrame;
  public
    destructor Destroy; override;

    procedure TimerCallback(TimerID: Cardinal);

    procedure Start(FramePeriod: Cardinal);
    procedure Stop;
    property IsActive: Boolean read GetIsActive;
  published
    property OnPaintNextFrame: TNotifyEvent read FOnPaintNextFrame write FOnPaintNextFrame;
    property OnSetNextFrame: TNotifyEvent read FOnSetNextFrame write FOnSetNextFrame;
  end;

implementation

{ TWinPaintBox }                 

constructor TWinPaintBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csAcceptsControls, csReplicatable, csOpaque];
  Width := 105;
  Height := 105;
end;

procedure TWinPaintBox.WMEraseBkgnd(var Message: TMessage);
begin
  // verhindert Lschen des Hintergrundes und damit das Flimmern
  Message.Result := 1;
end;

procedure TWinPaintBox.InvalidateControl;
begin
  InvalidateRect(Handle, Nil, False);
end;

procedure TWinPaintBox.Paint;
begin
  Canvas.Font := Font;
  Canvas.Brush.Color := Color;
  if csDesigning in ComponentState then begin
    Canvas.Pen.Style := psDash;
    Canvas.Brush.Style := bsBDiagonal;
    SetBkMode(Canvas.Handle, OPAQUE);
    SetBkColor(Canvas.Handle, ColorToRGB(clBtnShadow));
    Canvas.Rectangle(0, 0, Width, Height);
  end
  else begin
    if Assigned(FOnPaint) then
      FOnPaint(Self)
    {else
      Canvas.FillRect(ClientRect)};
  end;
end;

// *******************************************************************************************

// Callbackfunktion fr Wiedergabe
// dwUser: TAnimationWindow
procedure AnimationTimerCallback(uTimerID, uMessage: UINT; dwUser, dw1, dw2: DWORD); stdcall; 
begin
  if dwUser <> 0 then
    TAnimationPaintBox(dwUser).TimerCallback(uTimerID);
end;

destructor TAnimationPaintBox.Destroy;
begin
  // Wiedergabe anhalten
  TimerStop;
  inherited Destroy;
end;

procedure TAnimationPaintBox.TimerCallback(TimerID: Cardinal);
begin
  if FTimerID <> TimerID then   // falsche Timer-ID
    Exit;
  if not FPaintLock then begin  // nchste Frame zeichnen
    FPaintLock := True;
    PostMessage(Handle, WM_PaintNextFrame, 0, 0);
  end
  else                          // nur Pointer weitersetzen
    PostMessage(Handle, WM_SetNextFrame, 0, 0);
end;

procedure TAnimationPaintBox.TimerStart(FramePeriod: Cardinal);
begin
  // Timer starten
  if (FTimerID = 0) then
    FTimerID := timeSetEvent(FramePeriod, FramePeriod, TFNTimeCallBack(@AnimationTimerCallback),
        LongInt(Self), TIME_PERIODIC);
end;

procedure TAnimationPaintBox.TimerStop;
begin
  if IsActive then
    timeKillEvent(FTimerID);
  FTimerID := 0;
end;

function TAnimationPaintBox.GetIsActive: Boolean;
begin
  Result := FTimerID <> 0;
end;

procedure TAnimationPaintBox.WMPaintNextFrame(var Message: TMessage);
begin
  if IsActive then begin
    if Assigned(FOnPaintNextFrame) then
      FOnPaintNextFrame(Self)
    else
      InvalidateControl;
  end;
  FPaintLock := False;
end;

procedure TAnimationPaintBox.WMSetNextFrame(var Message: TMessage);
begin
  if IsActive then begin
    if Assigned(FOnSetNextFrame) then
      FOnSetNextFrame(Self);
  end;
end;

procedure TAnimationPaintBox.Start(FramePeriod: Cardinal);
begin
  TimerStart(FramePeriod);
end;

procedure TAnimationPaintBox.Stop;
begin
  TimerStop;
end;

// *******************************************************************************************

//  History:
//   2005-02-20, Peter J. Haas
//    - add license
//
//   2005-01-19, Peter J. Haas
//    - add TAnimationPaintBox
//
//   2002-06-09, Peter J. Haas
//    - delete FillRect
//
//   2002-01-14, Peter J. Haas
//    - add WMEraseBkgnd
//
//   1999-11-25, Peter J. Haas
//    - first version

end.
